
# q-expansion of newform 300.8.a.d, downloaded from the LMFDB on 07 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 300
weight = 8
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-27], [0], [1408], [-4044], [5890], [-31002], [-40300], [78912], [-157194], [114824], [471994], [-404310], [253852], [-437688], [-334926], [562596], [3246662], [-3895148], [-2345160], [-5726954], [-5222008], [2928132], [-3160230], [1898686], [-9829842], [-5891072], [22537164], [3291782], [-23374554], [-16564232], [34005612], [-1408626], [-29949556], [-39372738], [-12628240], [-31610654], [1746580], [-74141616], [32805018], [-83747844], [93211070], [-48070656], [-17222642], [30728802], [-181385728], [-108304060], [225721480], [-271001100], [-276697186], [215356110], [-412011648], [-42490222], [57304788], [-176317386], [-237052608], [100666374], [28976024], [338054746], [-336322374], [-33132404], [502960386], [619829044], [327307704], [-75755162], [889914618], [-475587028], [-778710626], [-299149476], [1281922694], [1065894246], [-778277544], [-766621832], [765360922], [-808440628], [242686296], [-1223989266], [114697954], [2388216786], [2734265834], [2138728908], [1462254110], [-2297834352], [-2511707522], [885268592], [-2547652212], [-1478846238], [2493746470], [-2780243034], [-2439501944], [-3092021196], [-4537517664], [1009249936], [-2812986156], [-2572288924], [3112573488], [-3299845962], [4623183642], [-2141781044], [222350678], [-3099215564], [5673748026], [2920893780], [-8372187126], [5150749340], [1049474686], [-5340260532], [-1571402778], [6493273704], [6405325418], [-7677300056], [-7850222534], [-218100018], [9023037356], [5587035872], [3298461714], [1924387348], [572440464], [1547632602], [3707828652], [-2935244722], [7090598350], [-369688782], [14101472748], [-8612158492], [-10074350634], [18191391326], [-630763008], [12094384096], [17893066882], [-12375535660], [-13074243792], [5290320632], [8944797082], [22577543082], [-786783262], [-2857662846], [-10917722924], [-14416991094], [10953612714], [-16187629780], [3296331006], [5473508656], [2648041836], [-9412933642], [645230616], [22445197834], [13743960366], [-3199095220], [8086868424], [-12536703758], [-5176443870], [28571851204], [14675621568], [2659211884], [35046691968], [23954123792], [25592910882], [4512553894], [5615805990], [35215191444], [23657169918], [-12786344240], [-8301430812], [48233805606], [-19164355104], [54456853208], [-22388058998], [-20298596206], [-51701525598], [-23773799868], [38070604022], [8892043992], [-17579963786], [-7050739864], [-52229911878], [65053341644], [-57978602610], [15626402800], [-60442035514], [-38589360968], [78728523036], [231799306], [78850927998], [25838096520], [30632602110], [-13192533998], [-21498697244], [73032360602], [-3229337376], [14584343182], [-12102997332], [-29050847164], [-53218868106], [-28444532556], [86790000222], [-56044301998], [22114051570], [29295455958], [56517607440], [96077460294], [-47374186456], [14449130986], [-97759210366], [-9966620316], [30270950778], [87506462216], [-33424664508], [-44521761174], [-42617262052], [-84057510146], [-129969643650], [-96317521808], [43652909868], [35131825704], [30699862010], [-113095617704], [-18716271054], [-98211262224], [-74086998678], [-58443353698], [-68737135504], [12713982450], [103717517128], [114345256500], [157424949566], [-140555322546], [-31358795472], [-75167585984], [-5440293660], [77844496642], [-15008811100], [83822702120], [-217643602902], [124552694476], [-188357485608], [-97976338510], [22747689282], [-125655156492], [-8827084008], [24057228468], [154696383548], [-142033387616], [242324053622], [95472523158], [31878451032], [161318105800], [45651851196], [-95946933220], [-111538662024], [104328522274], [77532769122], [175520627136], [-69831296518], [220186515306], [-8117309172], [-201886985218], [167914689148], [-231426130590], [-215351957018], [-259901850968], [-301029871164], [-63308639362], [245158065010], [302552476038], [73259257652], [80115199398], [141421221258], [39858459772], [113885010834], [-31976564794], [-140340215468], [26146010278], [305663966264], [9168172990], [-192378279056], [-310708515108], [-418556316730], [23307695738], [-171865241988], [-336464602152], [92962110992], [329057765952], [-350199220018], [190112049772], [183018696528], [-460043548562], [134489046498], [142382818304], [312783198786], [351272697606], [-307317461436], [-33180528930], [-216492934044], [232006318786], [631882997814], [340022719160], [-38969855310], [-110062253004], [-227248509932], [128485656886], [536438806266], [289300191512], [296086531572], [-604206252724], [-267304950626], [-203793896388], [-54240275530], [129411832056], [-342103649366], [525495558648], [204927554430], [-161585927070], [127485428260], [-332054033568], [-404911612678], [-105895860996], [-756742442832], [102718176430], [-71370807246], [-507866698540], [29031296134], [-388976203146], [11979764488], [-403811310786], [867103688594], [-380716674604], [-423313249076], [359969970120], [143360388786], [-128683513066], [788923383978], [297334445096], [-78042163260], [-378631797892], [830500124748], [-204105501418], [-336085920378], [541202753402], [-476813088056], [308999225818], [-391275268530], [-242683217298], [-988650151888], [-847466293542], [415478899068], [-394682111458], [-269637855428], [75375133440], [281818379634], [-645367153660], [33069655078], [-422199239994], [-218880465704], [293288598830], [-19313787426], [374727349248], [-216986906652], [866654712630], [-115085868336], [-860304197366], [-941125588326], [-468893020680], [325299850548], [1298810599540], [-937423064570], [515793568746], [-900528829904], [-401315433910], [-300893929140], [-66842123716], [-894843489432], [907993965918], [-1172110428688], [552065888770], [777716156556], [251451025488], [-1299218668994], [-1019252956590], [-416776156490], [111870672054], [-969786283698], [157533556048], [605154114726], [700596158036], [1630555979904], [-143116552], [-1808906835398], [-260810755076], [-1459471967736], [-1129698836734], [262155418578], [-1723329876300], [156248908756], [-1238774221080], [-1112032347146], [1203105719480], [1801340996106], [1409015617532], [1262457109182], [762649302902], [894256560390], [-633460233656], [-358995841410], [150723833646], [-304919709136], [487644597874], [-537579679182], [666974268196], [-1498261377876], [-1466915418194], [-456796862718], [1276995964188], [-454812385900], [2217130902070], [-336417551610], [-103660929024], [1832657519296], [-2153782636074], [-1223120125056], [903323079414], [525686024362], [752017444152], [1636870970628], [-957866472362], [-504051434646], [2531607824196], [-713336489718], [1422560727020], [1640096841480]]
